
 { Unit for Triple Buffering work with 13h VGA mode }
 { and some effects:                                }
 { transparent motion blur / motion blur            }
 { bilinear, trilinear, bicubic filtering           }
 { Code by VanDamM / [WRC]                          }

Unit V_Sharp;
interface

Type
 DoubleBuffer = array[0..319, 0..199] of byte;
 DBufferPtr   = ^DoubleBuffer;

Var
 DBuffer      : DBufferPtr;
 Screen_13h   : DBufferPtr;
 BackGround   : DBufferPtr;
 SBuffer      : DBufferPtr;

Procedure init13h;
Procedure kill13h;

Procedure initVariables;
Procedure killVariables;

Procedure PutPixel(WhereDraw : DBufferPtr; X, Y : integer; Color : byte);
Function  GetPixel(WhereDraw : DBufferPtr; X, Y : integer) : byte;

Procedure WaitRetrace;

Procedure Motion_Blur(Var D; Step : Byte);
Procedure Transparent_Motion_Blur(Step : Byte);

Procedure Bilinear_Filtering (Var WorkingPage);
Procedure Trilinear_Filtering(Var WorkingPage);
Procedure Bicubic_Filtering  (Var WorkingPage);

implementation

Procedure init13h; Assembler;
Asm
 mov ax, 0013h
 int 10h
End;

Procedure kill13h; Assembler;
Asm
 mov ax, 0003h
 int 10h
End;

Procedure initVariables;
Begin
 New(DBuffer);
 New(BackGround);
 New(SBuffer);
End;

Procedure killVariables;
Begin
 Dispose(DBuffer);
 Dispose(BackGround);
 Dispose(SBuffer);
End;

Procedure PutPixel(WhereDraw : DBufferPtr; X, Y : integer; Color : byte);
Var
 SOffset, R : longint;
Begin
 R:=Y; SOffset:= R shl 8 + R shl 6 + X;
 mem[Seg(WhereDraw^):SOffset]:=Color;
End;

Function  GetPixel(WhereDraw : DBufferPtr; X, Y : integer) : byte;
Var
 SOffset, R : longint;
Begin
 R:=Y; SOffset:= R shl 8 + R shl 6 + X;
 GetPixel:=mem[Seg(WhereDraw^):SOffset];
End;

Procedure WaitRetrace;
Begin
 Repeat Until ((Port[$3DA] AND 8) <> 0);
End;

Procedure Bilinear_Filtering(Var WorkingPage); assembler;
Asm
 push ds
 lds di, WorkingPage
 add di, (320+1) + 60
 mov cx, 198
@001:
 push cx
 push di
 mov cx, 320
@002:
 sub ax, ax
 sub bx, bx
 mov al, [di-1]
 add bx, ax
 mov al, [di+1]
 add bx, ax
 mov al, [di-320]
 add bx, ax
 mov al, [di+320]
 add bx, ax

 shr bx,2

 mov al, [di]
 add bx, ax
 shr bx, 1

 mov [di], bl
 inc di
 dec cx
 cmp cx, 0
 jne @002

 pop di
 pop cx
 add di, 320
 dec cx
 cmp cx, 0
 jne @001
 pop ds
End;

Procedure Trilinear_Filtering(Var WorkingPage); assembler;
Asm
 push ds
 lds di, WorkingPage
 add di, (320+1) + 60
 mov cx, (200-2)
@001:
 push cx
 push di
 mov cx, 320
@002:
 mov ax, 0
 mov bx, 0
 mov al, [di-1]
 add bx, ax
 mov al, [di+1]
 add bx, ax
 mov al, [di-320]
 add bx, ax
 mov al, [di+320]
 add bx, ax
 mov al, [di-(320-1)]
 add bx, ax
 mov al, [di-(320-1)]
 add bx, ax
 mov al, [di+(320+1)]
 add bx, ax
 mov al, [di+(320-1)]
 add bx, ax
 shr bx, 3
 mov al, [di]
 add bx, ax
 shr bx, 1
 mov [di], bl
 inc di
 dec cx
 cmp cx, 0
 jne @002
 pop di
 pop cx
 add di, 320
 dec cx
 cmp cx, 0
 jne @001
 pop ds
End;

Procedure Bicubic_Filtering(Var WorkingPage); assembler;
Asm
 push ds
 lds di, WorkingPage
 add di, (320+1)+60
 mov cx, (200-2)
@001:
 push cx
 push di
 mov cx, 320
@002:
 sub ax, ax
 sub dx, dx
 mov al, [di-1]
 add dx, ax
 mov al, [di+1]
 add dx, ax
 mov al, [di-320]
 add dx, ax
 mov al, [di+320]
 add dx, ax

 mov al, [di-(320+1)]
 add dx, ax
 mov al, [di-(320-1)]
 add dx, ax
 mov al, [di+(320+1)]
 add dx, ax
 mov al, [di+(320-1)]
 add dx, ax

 shr dx, 3

 mov al, [di]
 add dx, ax
 shr dx, 1

 mov [di-(320+1)], dl
 mov [di+(320+1)], dl
 mov [di-(320-1)], dl
 mov [di+(320-1)], dl

 inc di
 dec cx
 cmp cx, 0
 jg @002

 pop di
 pop cx

 add di, 320
 dec cx
 cmp cx, 0
 jg @001
 pop ds
End;

Procedure Motion_Blur(Var D; Step : Byte); Assembler;
Asm
  les di, D
  mov bl, Step
  mov cx, 64000
@001:
  mov al, es:[di]
  sub al, bl
  jns @002
  mov al, 0
@002:
  mov es:[di], al
  inc di
  dec cx
  jnz @001
End;

Procedure Transparent_Motion_Blur(Step : Byte);
Var DBufSeg: word;
    BackSeg: word;
    ScreenSeg: word;
    SecondSeg: word;
Begin

 DBufSeg:=Seg(DBuffer^);
 BackSeg:=Seg(BackGround^);
 ScreenSeg:=Seg(Screen_13h^);
 SecondSeg:=Seg(SBuffer^);

asm

  mov di, 64000
  mov es, SecondSeg
  push ds
  mov ds, DBufSeg
  xor cx, cx

@001:

  mov ax, [di]
  mov dx, es:[di]
  mov bx, dx

  and al, al
  jz  @002
  add bl, al
  rcr bl, 1
  mov [di], cl

@002:

  sub bl, Step
  jnc @003
  xor bl, bl

@003:

  and ah, ah
  jz  @004
  add bh, ah
  rcr bh, 1
  mov [di+1], cl

@004:

  sub bh, Step
  jnc @005
  xor bh, bh

@005:

  mov es:[di], bx

  push ds
  push es
  mov ds, BackSeg
  mov es, ScreenSeg
  add bx, ds:[di]
  mov es:[di], bx
  pop es
  pop ds

  sub di, 2
  jnc @001
  pop ds

end;
End;

Begin
 Screen_13h := Ptr($A000,$0000);
End.